from enum import Enum

VERSIONS = ("1.0", "1.1", "1.2")
SCHEMA = "http://adaptivecards.io/schemas/adaptive-card.json"


class FallbackOption(str, Enum):
    DROP = "drop"


class Style(str, Enum):
    DEFAULT = "default"
    EMPHASIS = "emphasis"
    GOOD = "good"
    ATTENTION = "attention"
    WARNING = "warning"
    ACCENT = "accent"


class HorizontalAlignment(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class VerticalAlignment(str, Enum):
    TOP = "top"
    CENTER = "center"
    BOTTOM = "bottom"


class Color(str, Enum):
    DEFAULT = "default"
    DARK = "dark"
    LIGHT = "light"
    ACCENT = "accent"
    GOOD = "good"
    WARNING = "warning"
    ATTENTION = "attention"


class FontType(str, Enum):
    DEFAULT = "default"
    MONOSPACE = "monospace"


class FontSize(str, Enum):
    DEFAULT = "default"
    SMALL = "small"
    MEDIUM = "medium"
    LARGE = "large"
    EXTRA_LARGE = "extraLarge"


class FontWeight(str, Enum):
    DEFAULT = "default"
    LIGHTER = "lighter"
    BOLDER = "bolder"


class BlockElementHeight(str, Enum):
    AUTO = "auto"
    STRETCH = "stretch"


class SpacingStyle(str, Enum):
    DEFAULT = "default"
    NONE = "none"
    SMALL = "small"
    MEDIUM = "medium"
    LARGE = "large"
    EXTRA_LARGE = "extraLarge"
    PADDING = "padding"


class ImageSize(str, Enum):
    AUTO = "auto"
    STRETCH = "stretch"
    SMALL = "small"
    MEDIUM = "medium"
    LARGE = "large"


class ImageStyle(str, Enum):
    DEFAULT = "default"
    PERSON = "person"


class ActionStyle(str, Enum):
    DEFAULT = "default"
    POSITIVE = "positive"
    DESTRUCTIVE = "destructive"


class ChoiceInputStyle(str, Enum):
    COMPACT = "compact"
    EXPANDED = "expanded"


class TextInputStyle(str, Enum):
    TEXT = "text"
    TEL = "tel"
    URL = "url"
    EMAIL = "email"


class Width(str, Enum):
    AUTO = "auto"
    STRETCH = "stretch"


class FillMode(str, Enum):
    COVER = "cover"
    REPEAT_HORIZONTALLY = "repeatHorizontally"
    REPEAT_VERTICALLY = "repeatVertically"
    REPEAT = "repeat"



from typing import List

from django_actionable_messages.message_card.elements import InputChoice
from django_actionable_messages.message_card.utils import ChoiceStyle
from django_actionable_messages.utils import CardElement, CardException


class InputMixin(CardElement):
    input_type = None

    def __init__(self, input_id: str = None, title: str = None, value: str = None, is_required: bool = None, **kwargs):
        self._data = {
            "@type": self.input_type
        }
        if input_id is not None:
            self.set_id(input_id)
        if title is not None:
            self.set_title(title)
        if value is not None:
            self.set_value(value)
        if is_required is not None:
            self.set_is_required(is_required)
        super().__init__()

    def set_id(self, input_id: str):
        self._data["id"] = input_id

    def set_title(self, title: str):
        self._data["title"] = title

    def set_value(self, value: str):
        self._data["value"] = value

    def set_is_required(self, is_required=True):
        self._data["isRequired"] = is_required


class TextInput(InputMixin):
    input_type = "TextInput"

    def __init__(self, max_length: int = None, is_multiline: bool = False, **kwargs):
        super().__init__(**kwargs)
        if max_length is not None:
            self.set_max_length(max_length)
        self.set_is_multiline(is_multiline)

    def set_max_length(self, max_length: int):
        self._data["maxLength"] = max_length

    def set_is_multiline(self, is_multiline=True):
        self._data["isMultiline"] = is_multiline


class DateInput(InputMixin):
    input_type = "DateInput"

    def __init__(self, include_time: bool = None, **kwargs):
        super().__init__(**kwargs)
        if include_time is not None:
            self.set_include_time(include_time)

    def set_include_time(self, include_time=True):
        self._data["includeTime"] = include_time


class MultiChoiceInput(InputMixin):
    input_type = "MultichoiceInput"

    def __init__(self, choices: List[InputChoice] = None, is_multi_select: bool = None, style: ChoiceStyle = None,
                 **kwargs):
        super().__init__(**kwargs)
        if choices:
            self.add_choices(choices)
        if is_multi_select is not None:
            self.set_is_multi_select(is_multi_select)
        if style is not None:
            self.set_style(style)

    def add_choices(self, choices: List[InputChoice]):
        for choice in choices:
            self.add_choice(choice)

    def add_choice(self, choice: InputChoice):
        choices = self._data.setdefault("choices", [])
        self._check_choice(choice, choices)
        self._data["choices"].append(choice.as_data())

    def set_is_multi_select(self, is_multi_select=True):
        self._data["isMultiSelect"] = is_multi_select

    def set_style(self, style: ChoiceStyle):
        self._data["style"] = style.value

    def _check_choice(self, choice, choices):
        choice_value = choice.get_value()
        values_list = (c["value"] for c in choices)
        if choice_value in values_list:
            raise CardException("Choice with this 'value' [{}] already added".format(choice_value))



from typing import Union

from django_actionable_messages.adaptive_card.utils import FallbackOption, ActionStyle
from django_actionable_messages.utils import CardElement, CardException


class ActionMixin(CardElement):
    def __init__(self, title: str = None, icon_url: str = None, style: ActionStyle = None, fallback=None,
                 requires: dict = None, **kwargs):
        if title is not None:
            self.set_title(title)
        if icon_url is not None:
            self.set_icon_url(icon_url)
        if style is not None:
            self.set_style(style)
        if fallback is not None:
            self.set_fallback(fallback)
        if requires is not None:
            self.set_requires(requires)
        super().__init__()

    def set_title(self, title: str):
        self._data["title"] = title

    def set_icon_url(self, url: str):
        self._data["iconUrl"] = url

    def set_style(self, style: ActionStyle):
        self._data["style"] = style.value

    def set_fallback(self, fallback):
        if isinstance(fallback, FallbackOption):
            self._data["fallback"] = fallback.value
        elif isinstance(fallback, ActionMixin):
            self._data["fallback"] = fallback.as_data()
        else:
            raise CardException("Invalid fallback type")

    def set_requires(self, requires: dict):
        self._data["requires"] = requires


class OpenUrl(ActionMixin):
    def __init__(self, url: str, **kwargs):
        self._data = {
            "type": "Action.OpenUrl",
            "url": url
        }
        super().__init__(**kwargs)

    def set_url(self, url: str):
        self._data["url"] = url


class Submit(ActionMixin):
    def __init__(self, data: Union[str, dict] = None, **kwargs):
        self._data = {
            "type": "Action.Submit"
        }
        if data is not None:
            self.set_data(data)
        super().__init__(**kwargs)

    def set_data(self, data: Union[str, dict]):
        self._data["data"] = data


class ShowCard(ActionMixin):
    def __init__(self, card=None, **kwargs):
        self._data = {
            "type": "Action.ShowCard"
        }
        if card is not None:
            self.set_card(card)
        super().__init__(**kwargs)

    def set_card(self, card):
        self._data["card"] = card.payload


class TargetElement(CardElement):
    def __init__(self, element_id: str, is_visible: bool = None):
        self._data = {
            "elementId": element_id
        }
        if is_visible is not None:
            self.set_is_visible(is_visible)
        super().__init__()

    def set_element_id(self, element_id: str):
        self._data["elementId"] = element_id

    def set_is_visible(self, value=True):
        self._data["isVisible"] = value


class ToggleVisibility(ActionMixin):
    def __init__(self, target_elements: list = None, **kwargs):
        self._data = {
            "type": "Action.ToggleVisibility"
        }
        if target_elements:
            self.set_target_elements(target_elements)
        super().__init__(**kwargs)

    def set_target_elements(self, elements: list):
        self._data["targetElements"] = []
        for element in elements:
            if isinstance(element, TargetElement):
                self._data["targetElements"].append(element.as_data())
            elif isinstance(element, str):
                self._data["targetElements"].append(element)
            else:
                raise CardException("Invalid target element type")



from typing import Union

from django_actionable_messages.adaptive_card.elements import Image
from django_actionable_messages.adaptive_card.utils import VERSIONS, Style, VerticalAlignment
from django_actionable_messages.utils import ADAPTIVE_CARD, Card, CardException


class AdaptiveCard(Card):
    card_type = ADAPTIVE_CARD

    def __init__(self, version: str = None, schema: str = None, inputs: list = None, actions: list = None,
                 select_action=None, style: Style = None, hide_original_body: bool = None, fallback_text: str = None,
                 background_image: Union[str, Image] = None, min_height: str = None, speak: str = None,
                 lang: str = None, vertical_content_alignment: VerticalAlignment = None):
        self._payload = {
            "type": "AdaptiveCard"
        }
        if version is not None:
            self.set_version(version)
        if schema is not None:
            self.set_schema(schema)
        if inputs:
            self.add_elements(inputs)
        if actions:
            self.add_actions(actions)
        if select_action is not None:
            self.set_select_action(select_action)
        if style is not None:
            self.set_style(style)
        if hide_original_body is not None:
            self.set_hide_original_body(hide_original_body)
        if fallback_text is not None:
            self.set_fallback_text(fallback_text)
        if background_image is not None:
            self.set_background_image(background_image)
        if min_height is not None:
            self.set_min_height(min_height)
        if speak is not None:
            self.set_speak(speak)
        if lang is not None:
            self.set_lang(lang)
        if vertical_content_alignment is not None:
            self.set_vertical_content_alignment(vertical_content_alignment)
        super().__init__()

    def set_version(self, version: str):
        if version not in VERSIONS:
            raise CardException("Invalid version. Supported versions are: {}".format(VERSIONS))
        self._payload["version"] = version

    def set_schema(self, schema: str):
        self._payload["$schema"] = schema

    def set_select_action(self, action):
        self._payload["selectAction"] = action.as_data()

    def set_style(self, style: Style):
        self._payload["style"] = style.value

    def set_hide_original_body(self, value=True):
        self._payload["hideOriginalBody"] = value

    def set_fallback_text(self, text: str):
        self._payload["fallbackText"] = text

    def set_background_image(self, image: Union[str, Image]):
        if isinstance(image, Image):
            self._payload["backgroundImage"] = image.as_data()
        elif isinstance(image, str):
            self._payload["backgroundImage"] = image
        else:
            raise CardException("Invalid image type")

    def set_min_height(self, height: str):
        self._payload["minHeight"] = height

    def set_speak(self, text: str):
        self._payload["speak"] = text

    def set_lang(self, lang: str):
        self._payload["lang"] = lang

    def set_vertical_content_alignment(self, alignment: VerticalAlignment):
        self._payload["verticalContentAlignment"] = alignment.value

    def add_element(self, element):
        self._payload.setdefault("body", [])
        self._payload["body"].append(element.as_data())

    def add_elements(self, elements: list):
        self._payload.setdefault("body", [])
        self._payload["body"].extend(self._get_items_list(elements))

    def add_action(self, action):
        self._payload.setdefault("actions", [])
        self._payload["actions"].append(action.as_data())

    def add_actions(self, actions: list):
        self._payload.setdefault("actions", [])
        self._payload["actions"].extend(self._get_items_list(actions))



import uuid
from typing import List

from django_actionable_messages.utils import MESSAGE_CARD, Card


class MessageCard(Card):
    card_type = MESSAGE_CARD

    def __init__(self, title: str = None, text: str = None, originator: str = None, summary: str = None,
                 theme_color: str = None, correlation_id: str = None, auto_correlation_id=True,
                 expected_actors: List[str] = None, hide_original_body: bool = None, sections: list = None,
                 actions: list = None):
        self._payload = {
            "@type": "MessageCard",
            "@context": "https://schema.org/extensions"
        }

        if title is not None:
            self.set_title(title)
        if text is not None:
            self.set_text(text)
        if originator is not None:
            self.set_originator(originator)
        if summary is not None:
            self.set_summary(summary)
        if theme_color is not None:
            self.set_theme_color(theme_color)
        if correlation_id is not None or auto_correlation_id:
            self._payload["correlationId"] = correlation_id or str(uuid.uuid4())
        if expected_actors:
            self.set_expected_actors(expected_actors)
        if hide_original_body is not None:
            self.set_hide_original_body(hide_original_body)
        if sections:
            self.add_sections(sections)
        if actions:
            self.add_actions(actions)
        super().__init__()

    def set_title(self, title: str):
        self._payload["title"] = title

    def set_text(self, text: str):
        self._payload["text"] = text

    def set_originator(self, originator: str):
        self._payload["originator"] = originator

    def set_summary(self, summary: str):
        self._payload["summary"] = summary

    def set_theme_color(self, theme_color: str):
        self._payload["themeColor"] = theme_color

    def set_correlation_id(self, correlation_id: str):
        self._payload["correlationId"] = correlation_id

    def set_expected_actors(self, expected_actors: List[str]):
        self._payload["expectedActors"] = expected_actors

    def set_hide_original_body(self, hide_original_body=True):
        self._payload["hideOriginalBody"] = hide_original_body

    def add_sections(self, sections: list):
        self._payload.setdefault("sections", [])
        self._payload["sections"].extend(self._get_items_list(sections))

    def add_section(self, section):
        self._payload.setdefault("sections", [])
        self._payload["sections"].append(section.as_data())

    def add_actions(self, actions: list):
        self._payload.setdefault("potentialAction", [])
        self._payload["potentialAction"].extend(self._get_items_list(actions))

    def add_action(self, action):
        self._payload.setdefault("potentialAction", [])
        self._payload["potentialAction"].append(action.as_data())



from typing import List

from django_actionable_messages.message_card.elements import Header, ActionTarget
from django_actionable_messages.utils import CardElement, CardException


class OpenUri(CardElement):
    def __init__(self, name: str, targets: List[ActionTarget] = None):
        self._data = {
            "@type": "OpenUri",
            "name": name
        }
        if targets:
            self.add_targets(targets)
        super().__init__()

    def set_name(self, name: str):
        self._data["name"] = name

    def add_targets(self, targets: List[ActionTarget]):
        for target in targets:
            self.add_target(target)

    def add_target(self, target: ActionTarget):
        targets = self._data.setdefault("targets", [])
        self._check_target(target, targets)
        self._data["targets"].append(target.as_data())

    def _check_target(self, target: ActionTarget, targets: list):
        os_type = target.get_os()
        os_list = (x["os"] for x in targets)
        if os_type in os_list:
            raise CardException("Target already set for '{}'".format(os_type))


class HttpPOST(CardElement):
    def __init__(self, name: str, target: str, headers: List[Header] = None, body: str = None,
                 body_content_type: str = None):
        self._data = {
            "@type": "HttpPOST",
            "name": name,
            "target": target
        }
        if headers:
            self.add_headers(headers)
        if body is not None:
            self.set_body(body)
        if body_content_type is not None:
            self.set_body_content_type(body_content_type)
        super().__init__()

    def set_name(self, name: str):
        self._data["name"] = name

    def set_target(self, target: str):
        self._data["target"] = target

    def add_header(self, header: Header):
        self._data.setdefault("headers", [])
        self._data["headers"].append(header.as_data())

    def add_headers(self, headers: List[Header]):
        self._data.setdefault("headers", [])
        self._data["headers"].extend(self._get_items_list(headers))

    def set_body(self, body: str):
        self._data["body"] = body

    def set_body_content_type(self, content_type: str):
        self._data["bodyContentType"] = content_type


class InvokeAddInCommand(CardElement):
    def __init__(self, name: str, add_in_id: str, desktop_command_id: str, initialization_context: dict = None):
        self._data = {
            "@type": "InvokeAddInCommand",
            "name": name,
            "addInId": add_in_id,
            "desktopCommandId": desktop_command_id
        }
        if initialization_context:
            self.set_initialization_context(initialization_context)
        super().__init__()

    def set_name(self, name: str):
        self._data["name"] = name

    def set_add_in_id(self, add_in_id: str):
        self._data["addInId"] = add_in_id

    def set_desktop_command_id(self, cmd_it: str):
        self._data["desktopCommandId"] = cmd_it

    def set_initialization_context(self, context: dict):
        self._data["initializationContext"] = context


class ActionCard(CardElement):
    def __init__(self, name: str, inputs: list = None, actions: list = None):
        self._data = {
            "@type": "ActionCard",
            "name": name
        }
        if inputs:
            self.add_inputs(inputs)
        if actions:
            self.add_actions(actions)
        super().__init__()

    def set_name(self, name: str):
        self._data["name"] = name

    def add_inputs(self, action_inputs: list):
        self._data.setdefault("inputs", [])
        self._data["inputs"].extend(self._get_items_list(action_inputs))

    def add_input(self, action_input):
        self._data.setdefault("inputs", [])
        self._data["inputs"].append(action_input.as_data())

    def add_actions(self, actions: list):
        self._data.setdefault("actions", [])
        self._data["actions"].extend(self._get_items_list(actions))

    def add_action(self, action):
        self._data.setdefault("actions", [])
        self._data["actions"].append(action.as_data())



from typing import List, Union

from django_actionable_messages.adaptive_card.mixins import ElementMixin
from django_actionable_messages.adaptive_card.utils import ChoiceInputStyle, TextInputStyle
from django_actionable_messages.utils import CardElement


class DateTimeMixin(ElementMixin):
    base_type = None

    def __init__(self, max_value: str = None, min_value: str = None, placeholder: str = None, value: str = None,
                 **kwargs):
        self._data = {
            "type": self.base_type
        }
        if max_value is not None:
            self.set_max(max_value)
        if min_value is not None:
            self.set_min(min_value)
        if placeholder is not None:
            self.set_placeholder(placeholder)
        if value is not None:
            self.set_value(value)
        super().__init__(**kwargs)

    def set_max(self, value: str):
        self._data["max"] = value

    def set_min(self, value: str):
        self._data["min"] = value

    def set_placeholder(self, text: str):
        self._data["placeholder"] = text

    def set_value(self, value: str):
        self._data["value"] = value


class TextInput(ElementMixin):
    def __init__(self, is_multiline: bool = None, max_length: int = None, placeholder: str = None,
                 style: TextInputStyle = None, inline_action=None, value: str = None, **kwargs):
        self._data = {
            "type": "Input.Text"
        }
        if is_multiline is not None:
            self.set_is_multiline(is_multiline)
        if max_length is not None:
            self.set_max_length(max_length)
        if placeholder is not None:
            self.set_placeholder(placeholder)
        if style is not None:
            self.set_style(style)
        if inline_action is not None:
            self.set_inline_action(inline_action)
        if value is not None:
            self.set_value(value)
        super().__init__(**kwargs)

    def set_is_multiline(self, value=True):
        self._data["isMultiline"] = value

    def set_max_length(self, value: int):
        self._data["maxLength"] = value

    def set_placeholder(self, text: str):
        self._data["placeholder"] = text

    def set_style(self, style: TextInputStyle):
        self._data["style"] = style.value

    def set_inline_action(self, action):
        self._data["inlineAction"] = action.as_data()

    def set_value(self, value: str):
        self._data["value"] = value


class NumberInput(ElementMixin):
    def __init__(self, max_value: int = None, min_value: int = None, placeholder: str = None, value: int = None,
                 **kwargs):
        self._data = {
            "type": "Input.Number"
        }
        if max_value is not None:
            self.set_max(max_value)
        if min_value is not None:
            self.set_min(min_value)
        if placeholder is not None:
            self.set_placeholder(placeholder)
        if value is not None:
            self.set_value(value)
        super().__init__(**kwargs)

    def set_max(self, value: int):
        self._data["max"] = value

    def set_min(self, value: int):
        self._data["min"] = value

    def set_placeholder(self, text: str):
        self._data["placeholder"] = text

    def set_value(self, value: int):
        self._data["value"] = value


class DateInput(DateTimeMixin):
    base_type = "Input.Date"


class TimeInput(DateTimeMixin):
    base_type = "Input.Time"


class ToggleInput(ElementMixin):
    def __init__(self, title: str, value: str = None, value_off: str = None, value_on: str = None, wrap: bool = None,
                 **kwargs):
        self._data = {
            "type": "Input.Toggle",
            "title": title
        }
        if value is not None:
            self.set_value(value)
        if value_off is not None:
            self.set_value_off(value_off)
        if value_on is not None:
            self.set_value_on(value_on)
        if wrap is not None:
            self.set_wrap(wrap)
        super().__init__(**kwargs)

    def set_title(self, title: str):
        self._data["title"] = title

    def set_value(self, value: str):
        self._data["value"] = value

    def set_value_off(self, value: str):
        self._data["valueOff"] = value

    def set_value_on(self, value: str):
        self._data["valueOn"] = value

    def set_wrap(self, value=True):
        self._data["wrap"] = value


class InputChoice(CardElement):
    def __init__(self, title: str, value: Union[str, int], **kwargs):
        self._data = {
            "title": title,
            "value": value
        }
        super().__init__(**kwargs)


class ChoiceSetInput(ElementMixin):
    def __init__(self, choices: List[InputChoice], is_multi_select: bool = None, style: ChoiceInputStyle = None,
                 value: str = None, wrap: bool = None, **kwargs):
        self._data = {
            "type": "Input.ChoiceSet",
            "choices": self._get_items_list(choices)
        }
        if is_multi_select is not None:
            self.set_is_multi_select(is_multi_select)
        if style is not None:
            self.set_style(style)
        if value is not None:
            self.set_value(value)
        if wrap is not None:
            self.set_wrap(wrap)
        super().__init__(**kwargs)

    def set_choices(self, choices: List[InputChoice]):
        self._data["choices"] = self._get_items_list(choices)

    def set_is_multi_select(self, value=True):
        self._data["isMultiSelect"] = value

    def set_style(self, style: ChoiceInputStyle):
        self._data["style"] = style.value

    def set_value(self, value: str):
        self._data["value"] = value

    def set_wrap(self, value=True):
        self._data["wrap"] = value



from enum import Enum


class OSType(str, Enum):
    DEFAULT = "default"
    WINDOWS = "windows"
    IOS = "iOS"
    ANDROID = "android"


class ChoiceStyle(str, Enum):
    NORMAL = "normal"
    EXPANDED = "expanded"



from django.conf import settings
from django.core.signals import setting_changed
from django.utils.module_loading import import_string

SETTINGS_NAMESPACE = "ACTIONABLE_MESSAGES"
DEFAULTS = {
    "JSON_ENCODER": None
}


def import_from_string(value, name):
    try:
        return import_string(value)
    except ImportError:
        raise ImportError("Could not import '{}' setting".format(name))


def import_setting(value, name):
    if value is None:
        return None
    elif isinstance(value, str):
        return import_from_string(value, name)
    return value


class CardSettings:
    def __getattr__(self, attr):
        if attr not in DEFAULTS:
            raise AttributeError("Invalid setting: '{}'".format(attr))
        try:
            value = self.user_settings[attr]
        except KeyError:
            value = DEFAULTS[attr]
        return import_setting(value, attr)

    @property
    def user_settings(self):
        if not hasattr(self, "_user_settings"):
            self._user_settings = getattr(settings, SETTINGS_NAMESPACE, {})
        return self._user_settings

    def reload_user_settings(self):
        if hasattr(self, "_user_settings"):
            delattr(self, "_user_settings")


card_settings = CardSettings()


def reload_settings(*args, **kwargs):
    setting = kwargs["setting"]
    if setting == SETTINGS_NAMESPACE:
        card_settings.reload_user_settings()


setting_changed.connect(reload_settings)



from typing import Union, List

from django_actionable_messages.adaptive_card.elements import Image
from django_actionable_messages.adaptive_card.mixins import ElementMixin
from django_actionable_messages.adaptive_card.types import BackgroundImage
from django_actionable_messages.adaptive_card.utils import (
    FallbackOption, Style, VerticalAlignment, SpacingStyle, ImageSize, Width
)
from django_actionable_messages.utils import CardElement, CardException


class ActionSet(ElementMixin):
    def __init__(self, actions: list, **kwargs):
        self._data = {
            "type": "ActionSet",
            "actions": self._get_items_list(actions)
        }
        super().__init__(**kwargs)

    def add_actions(self, actions: list):
        self._data["actions"].extend(self._get_items_list(actions))

    def add_action(self, action):
        self._data["actions"].append(action.as_data())


class Container(ElementMixin):
    def __init__(self, items: list, select_action=None, style: Style = None,
                 vertical_content_alignment: VerticalAlignment = None, bleed: bool = None,
                 background_image: Union[str, BackgroundImage] = None, min_height: str = None, **kwargs):
        self._data = {
            "type": "Container",
            "items": self._get_items_list(items)
        }
        if select_action:
            self.set_select_action(select_action)
        if style is not None:
            self.set_style(style)
        if vertical_content_alignment:
            self.set_vertical_content_alignment(vertical_content_alignment)
        if bleed is not None:
            self.set_bleed(bleed)
        if background_image is not None:
            self.set_background_image(background_image)
        if min_height is not None:
            self.set_min_height(min_height)
        super().__init__(**kwargs)

    def add_items(self, items: list):
        self._data["items"].extend(self._get_items_list(items))

    def add_item(self, item):
        self._data["items"].append(item.as_data())

    def set_select_action(self, action):
        self._data["selectAction"] = action.as_data()

    def set_style(self, style: Style):
        self._data["style"] = style.value

    def set_vertical_content_alignment(self, alignment: VerticalAlignment):
        self._data["verticalContentAlignment"] = alignment.value

    def set_bleed(self, value=True):
        self._data["bleed"] = value

    def set_background_image(self, image: Union[str, BackgroundImage]):
        if isinstance(image, str):
            self._data["backgroundImage"] = image
        elif isinstance(image, BackgroundImage):
            self._data["backgroundImage"] = image.as_data()
        else:
            raise CardException("Invalid image type")

    def set_min_height(self, height: str):
        self._data["minHeight"] = height


class Column(CardElement):
    def __init__(self, items: list = None, background_image: Union[str, BackgroundImage] = None, bleed: bool = None,
                 fallback=None, min_height: str = None, separator: bool = None, spacing: SpacingStyle = None,
                 select_action=None, style: Style = None, vertical_content_alignment: VerticalAlignment = None,
                 width: Union[str, int, Width] = None, item_id: str = None, is_visible: bool = None,
                 requires: dict = None):
        self._data = {
            "type": "Column"
        }
        if items:
            self.add_items(items)
        if background_image is not None:
            self.set_background_image(background_image)
        if bleed is not None:
            self.set_bleed(bleed)
        if fallback is not None:
            self.set_fallback(fallback)
        if min_height is not None:
            self.set_min_height(min_height)
        if separator is not None:
            self.set_separator(separator)
        if spacing is not None:
            self.set_spacing(spacing)
        if select_action is not None:
            self.set_select_action(select_action)
        if style is not None:
            self.set_style(style)
        if vertical_content_alignment:
            self.set_vertical_content_alignment(vertical_content_alignment)
        if width is not None:
            self.set_width(width)
        if item_id is not None:
            self.set_id(item_id)
        if is_visible is not None:
            self.set_is_visible(is_visible)
        if requires is not None:
            self.set_requires(requires)
        super().__init__()

    def add_items(self, items: list):
        self._data.setdefault("items", [])
        self._data["items"].extend(self._get_items_list(items))

    def add_item(self, item):
        self._data.setdefault("items", [])
        self._data["items"].append(item.as_data())

    def set_background_image(self, image: Union[str, BackgroundImage]):
        if isinstance(image, str):
            self._data["backgroundImage"] = image
        elif isinstance(image, BackgroundImage):
            self._data["backgroundImage"] = image.as_data()
        else:
            raise CardException("Invalid image type")

    def set_bleed(self, value=True):
        self._data["bleed"] = value

    def set_fallback(self, fallback):
        if isinstance(fallback, FallbackOption):
            self._data["fallback"] = fallback.value
        elif isinstance(fallback, Column):
            self._data["fallback"] = fallback.as_data()
        else:
            raise CardException("Invalid fallback type")

    def set_min_height(self, height: str):
        self._data["minHeight"] = height

    def set_separator(self, value=True):
        self._data["separator"] = value

    def set_spacing(self, spacing: SpacingStyle):
        self._data["spacing"] = spacing.value

    def set_select_action(self, action):
        self._data["selectAction"] = action.as_data()

    def set_style(self, style: Style):
        self._data["style"] = style.value

    def set_vertical_content_alignment(self, alignment: VerticalAlignment):
        self._data["verticalContentAlignment"] = alignment.value

    def set_width(self, width: Union[str, int, Width]):
        if isinstance(width, Width):
            self._data["width"] = width.value
        elif isinstance(width, (str, int)):
            self._data["width"] = width
        else:
            raise CardException("Invalid width type")

    def set_id(self, item_id: str):
        self._data["id"] = item_id

    def set_is_visible(self, value=True):
        self._data["isVisible"] = value

    def set_requires(self, requires: dict):
        self._data["requires"] = requires


class ColumnSet(ElementMixin):
    def __init__(self, columns: List[Column] = None, select_action=None, style: Style = None, bleed: bool = None,
                 min_height: str = None, **kwargs):
        self._data = {
            "type": "ColumnSet"
        }
        if columns:
            self.add_columns(columns)
        if select_action is not None:
            self.set_select_action(select_action)
        if style is not None:
            self.set_style(style)
        if bleed is not None:
            self.set_bleed(bleed)
        if min_height is not None:
            self.set_min_height(min_height)
        super().__init__(**kwargs)

    def add_columns(self, columns: List[Column]):
        self._data.setdefault("columns", [])
        self._data["columns"].extend(self._get_items_list(columns))

    def add_column(self, column: Column):
        self._data.setdefault("columns", [])
        self._data["columns"].append(column.as_data())

    def set_select_action(self, action):
        self._data["selectAction"] = action.as_data()

    def set_style(self, style: Style):
        self._data["style"] = style.value

    def set_bleed(self, value=True):
        self._data["bleed"] = value

    def set_min_height(self, value: str):
        self._data["minHeight"] = value


class Fact(CardElement):
    def __init__(self, title: str, value: str):
        self._data = {
            "title": title,
            "value": value
        }
        super().__init__()


class FactSet(ElementMixin):
    def __init__(self, facts: List[Fact], **kwargs):
        self._data = {
            "type": "FactSet",
            "facts": self._get_items_list(facts)
        }
        super().__init__(**kwargs)

    def add_facts(self, facts: List[Fact]):
        self._data["facts"].extend(self._get_items_list(facts))

    def add_fact(self, fact: Fact):
        self._data["facts"].append(fact.as_data())


class ImageSet(ElementMixin):
    def __init__(self, images: List[Image], image_size: ImageSize = None, **kwargs):
        self._data = {
            "type": "ImageSet",
            "images": self._get_items_list(images)
        }
        if image_size is not None:
            self.set_image_size(image_size)
        super().__init__(**kwargs)

    def add_images(self, images: List[Image]):
        self._data["images"].extend(self._get_items_list(images))

    def add_image(self, image: Image):
        self._data["images"].append(image.as_data())

    def set_image_size(self, size: ImageSize):
        self._data["imageSize"] = size.value



from typing import Union, List

from django_actionable_messages.adaptive_card.mixins import BaseElementMixin, ElementMixin
from django_actionable_messages.adaptive_card.utils import (
    HorizontalAlignment, Color, FontType, FontSize, FontWeight, BlockElementHeight, ImageSize, ImageStyle
)
from django_actionable_messages.utils import CardElement, CardException


class TextBlock(ElementMixin):
    def __init__(self, text: str, color: Color = None, font_type: FontType = None,
                 horizontal_alignment: HorizontalAlignment = None, is_subtle: bool = None, max_lines: int = None,
                 size: FontSize = None, weight: FontWeight = None, wrap: bool = None, **kwargs):
        self._data = {
            "type": "TextBlock",
            "text": text
        }
        if color is not None:
            self.set_color(color)
        if font_type is not None:
            self.set_font_type(font_type)
        if horizontal_alignment is not None:
            self.set_horizontal_alignment(horizontal_alignment)
        if is_subtle is not None:
            self.set_is_subtle(is_subtle)
        if max_lines is not None:
            self.set_max_lines(max_lines)
        if size is not None:
            self.set_size(size)
        if weight is not None:
            self.set_weight(weight)
        if wrap is not None:
            self.set_wrap(wrap)
        super().__init__(**kwargs)

    def set_text(self, text: str):
        self._data["text"] = text

    def set_color(self, color: Color):
        self._data["color"] = color.value

    def set_font_type(self, font_type: FontType):
        self._data["fontType"] = font_type.value

    def set_horizontal_alignment(self, alignment: HorizontalAlignment):
        self._data["horizontalAlignment"] = alignment.value

    def set_is_subtle(self, value=True):
        self._data["isSubtle"] = value

    def set_max_lines(self, max_lines: int):
        self._data["maxLines"] = max_lines

    def set_size(self, size: FontSize):
        self._data["size"] = size.value

    def set_weight(self, weight: FontWeight):
        self._data["weight"] = weight.value

    def set_wrap(self, value=True):
        self._data["wrap"] = value


class Image(BaseElementMixin):
    def __init__(self, url: str, alternate_text: str = None, background_color: str = None,
                 height: Union[str, BlockElementHeight] = None, horizontal_alignment: HorizontalAlignment = None,
                 select_action=None, size: ImageSize = None, style: ImageStyle = None, width: str = None, **kwargs):
        self._data = {
            "type": "Image",
            "url": url
        }
        if alternate_text is not None:
            self.set_alternate_text(alternate_text)
        if background_color is not None:
            self.set_background_color(background_color)
        if height is not None:
            self.set_height(height)
        if horizontal_alignment is not None:
            self.set_horizontal_alignment(horizontal_alignment)
        if select_action is not None:
            self.set_select_action(select_action)
        if size is not None:
            self.set_size(size)
        if style is not None:
            self.set_style(style)
        if width is not None:
            self.set_width(width)
        super().__init__(**kwargs)

    def set_url(self, url: str):
        self._data["url"] = url

    def set_alternate_text(self, text: str):
        self._data["altText"] = text

    def set_background_color(self, color: str):
        self._data["backgroundColor"] = color

    def set_height(self, height: Union[str, BlockElementHeight]):
        if isinstance(height, BlockElementHeight):
            self._data["height"] = height.value
        elif isinstance(height, str):
            self._data["height"] = height
        else:
            raise CardException("Invalid height type")

    def set_horizontal_alignment(self, alignment: HorizontalAlignment):
        self._data["horizontalAlignment"] = alignment.value

    def set_select_action(self, action):
        self._data["selectAction"] = action.as_data()

    def set_size(self, size: ImageSize):
        self._data["size"] = size.value

    def set_style(self, style: ImageStyle):
        self._data["style"] = style.value

    def set_width(self, width: str):
        self._data["width"] = width


class MediaSource(CardElement):
    def __init__(self, mime_type: str, url: str):
        self._data = {
            "mimeType": mime_type,
            "url": url
        }
        super().__init__()


class Media(ElementMixin):
    def __init__(self, sources: List[MediaSource], poster: str = None, alternate_text: str = None, **kwargs):
        self._data = {
            "type": "Media",
            "sources": self._get_items_list(sources)
        }
        if poster is not None:
            self.set_poster(poster)
        if alternate_text is not None:
            self.set_alternate_text(alternate_text)
        super().__init__(**kwargs)

    def add_sources(self, sources: List[MediaSource]):
        self._data["sources"].extend(self._get_items_list(sources))

    def add_source(self, source: MediaSource):
        self._data["sources"].append(source.as_data())

    def set_poster(self, poster: str):
        self._data["poster"] = poster

    def set_alternate_text(self, text: str):
        self._data["altText"] = text


class TextRun(CardElement):
    def __init__(self, text: str, color: Color = None, font_type: FontType = None, highlight: bool = None,
                 is_subtle: bool = None, italic: bool = None, select_action=None, size: FontSize = None,
                 strike_through: bool = None, weight: FontWeight = None):
        self._data = {
            "type": "TextRun",
            "text": text
        }
        if color is not None:
            self.set_color(color)
        if font_type is not None:
            self.set_font_type(font_type)
        if highlight is not None:
            self.set_highlight(highlight)
        if is_subtle is not None:
            self.set_is_subtle(is_subtle)
        if italic is not None:
            self.set_italic(italic)
        if select_action:
            self.set_select_action(select_action)
        if size is not None:
            self.set_size(size)
        if strike_through is not None:
            self.set_strike_through(strike_through)
        if weight is not None:
            self.set_weight(weight)
        super().__init__()

    def set_text(self, text: str):
        self._data["text"] = text

    def set_color(self, color: Color):
        self._data["color"] = color.value

    def set_font_type(self, font_type: FontType):
        self._data["fontType"] = font_type.value

    def set_highlight(self, value=True):
        self._data["highlight"] = value

    def set_is_subtle(self, value=True):
        self._data["isSubtle"] = value

    def set_italic(self, value=True):
        self._data["italic"] = value

    def set_select_action(self, action):
        self._data["selectAction"] = action.as_data()

    def set_size(self, size: FontSize):
        self._data["size"] = size.value

    def set_strike_through(self, value=True):
        self._data["strikethrough"] = value

    def set_weight(self, weight: FontWeight):
        self._data["weight"] = weight.value


class RichTextBlock(ElementMixin):
    def __init__(self, inlines: list, horizontal_alignment: HorizontalAlignment = None,
                 **kwargs):
        self._data = {
            "type": "RichTextBlock"
        }
        self.set_inlines(inlines)
        if horizontal_alignment is not None:
            self.set_horizontal_alignment(horizontal_alignment)
        super().__init__(**kwargs)

    def set_inlines(self, inlines: list):
        self._data["inlines"] = []
        for item in inlines:
            if isinstance(item, str):
                self._data["inlines"].append(item)
            elif isinstance(item, TextRun):
                self._data["inlines"].append(item.as_data())
            else:
                raise CardException("Invalid inline type")

    def set_horizontal_alignment(self, alignment: HorizontalAlignment):
        self._data["horizontalAlignment"] = alignment.value



import copy
import json

from django.template.loader import render_to_string

from django_actionable_messages.settings import card_settings

MESSAGE_CARD = 1
ADAPTIVE_CARD = 2


def get_items_list(items):
    return list([item.as_data() for item in items])


class CardElement:
    _data = None

    def as_data(self):
        return self._data

    def _get_items_list(self, items):
        return get_items_list(items)


class Card:
    json_encoder = card_settings.JSON_ENCODER

    _payload = None
    card_type = None
    script_types = {
        MESSAGE_CARD: "application/ld+json",
        ADAPTIVE_CARD: "application/adaptivecard+json"
    }

    @property
    def payload(self):
        return self.get_payload()

    @property
    def json_payload(self):
        return self.get_payload(fmt="json")

    @property
    def html_payload(self):
        return self.get_payload(fmt="html")

    def get_payload(self, fmt=None):
        payload = copy.deepcopy(self._payload)
        if fmt == "json":
            payload = self._get_json_payload(payload)
        elif fmt == "html":
            payload = self._get_html_payload(payload)
        return payload

    def _get_json_payload(self, payload):
        return json.dumps(payload, default=self.json_encoder)

    def _get_html_payload(self, payload):
        context = {
            "script_type": self.script_types[self.card_type],
            "payload": self._get_json_payload(payload)
        }
        return render_to_string("django_actionable_messages/email.html", context=context)

    def _get_items_list(self, items):
        return get_items_list(items)


class CardException(Exception):
    pass



from typing import Union

from django_actionable_messages.message_card.utils import OSType
from django_actionable_messages.utils import CardElement


class Header(CardElement):
    def __init__(self, name: str, value: Union[str, int]):
        self._data = {
            "name": name,
            "value": value
        }
        super().__init__()

    def get_name(self):
        return self._data["name"]


class Fact(CardElement):
    def __init__(self, name: str, value: str):
        self._data = {
            "name": name,
            "value": value
        }
        super().__init__()


class HeroImage(CardElement):
    def __init__(self, url: str, title: str = None):
        self._data = {
            "image": url
        }
        if title is not None:
            self.set_title(title)
        super().__init__()

    def set_url(self, url: str):
        self._data["image"] = url

    def set_title(self, title: str):
        self._data["title"] = title


class InputChoice(CardElement):
    def __init__(self, name: str, value: Union[str, int]):
        self._data = {
            "display": name,
            "value": value
        }
        super().__init__()

    def get_value(self):
        return self._data["value"]


class ActionTarget(CardElement):
    def __init__(self, os_type: OSType, url: str):
        self._data = {
            "os": os_type.value,
            "uri": url
        }
        super().__init__()

    def get_os(self):
        return self._data["os"]



from django_actionable_messages.adaptive_card.utils import FallbackOption, BlockElementHeight, SpacingStyle
from django_actionable_messages.utils import CardElement, CardException


class BaseElementMixin(CardElement):
    def __init__(self, fallback=None, separator: bool = None, spacing: SpacingStyle = None, item_id: str = None,
                 is_visible: bool = None, requires: dict = None, **kwargs):
        if fallback is not None:
            self.set_fallback(fallback)
        if separator is not None:
            self.set_separator(separator)
        if spacing is not None:
            self.set_spacing(spacing)
        if item_id is not None:
            self.set_id(item_id)
        if is_visible is not None:
            self.set_is_visible(is_visible)
        if requires is not None:
            self.set_requires(requires)
        super().__init__()

    def set_fallback(self, fallback):
        if isinstance(fallback, FallbackOption):
            self._data["fallback"] = fallback.value
        elif isinstance(fallback, BaseElementMixin):
            self._data["fallback"] = fallback.as_data()
        else:
            raise CardException("Invalid fallback type")

    def set_separator(self, value=True):
        self._data["separator"] = value

    def set_spacing(self, spacing: SpacingStyle):
        self._data["spacing"] = spacing.value

    def set_id(self, item_id: str):
        self._data["id"] = item_id

    def set_is_visible(self, visible=True):
        self._data["isVisible"] = visible

    def set_requires(self, requires: dict):
        self._data["requires"] = requires


class ElementMixin(BaseElementMixin):
    def __init__(self, height: BlockElementHeight = None, **kwargs):
        if height is not None:
            self.set_height(height)
        super().__init__(**kwargs)

    def set_height(self, height: BlockElementHeight):
        self._data["height"] = height.value



from django_actionable_messages.adaptive_card.utils import HorizontalAlignment, VerticalAlignment, FillMode
from django_actionable_messages.utils import CardElement


class BackgroundImage(CardElement):
    def __init__(self, url: str, fill_mode: FillMode = None, horizontal_alignment: HorizontalAlignment = None,
                 vertical_alignment: VerticalAlignment = None, **kwargs):
        self._data = {
            "url": url
        }
        if fill_mode is not None:
            self.set_fill_mode(fill_mode)
        if horizontal_alignment is not None:
            self.set_horizontal_alignment(horizontal_alignment)
        if vertical_alignment is not None:
            self.set_vertical_alignment(vertical_alignment)
        super().__init__(**kwargs)

    def set_url(self, url: str):
        self._data["url"] = url

    def set_fill_mode(self, fill_mode: FillMode):
        self._data["fillMode"] = fill_mode.value

    def set_horizontal_alignment(self, alignment: HorizontalAlignment):
        self._data["horizontalAlignment"] = alignment.value

    def set_vertical_alignment(self, alignment: VerticalAlignment):
        self._data["verticalAlignment"] = alignment.value



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="django_actionable_messages",
    packages=find_packages(exclude=["examples", "examples.*", "tests", "tests.*"]),
    include_package_data=True,
    version="0.1.1",
    license="MIT",
    description="Actionable messages",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="utsurius",
    author_email="przemek@upsecure.pl",
    url="https://github.com/utsurius/django-actionable-messages",
    python_requires=">=3.5",
    keywords=["msteams", "AdaptiveCard", "MessageCard", "actionable messages"],
    install_requires=[
        "django>=1.11.0",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    project_urls={
        "Documentation": "https://github.com/utsurius/django-actionable-messages/README.md",
        "Source": "https://github.com/utsurius/django-actionable-messages",
        "Tracker": "https://github.com/utsurius/django-actionable-messages/issues",
    }
)



from typing import List

from django_actionable_messages.message_card.elements import Fact, HeroImage
from django_actionable_messages.utils import CardElement


class Section(CardElement):
    def __init__(self, start_group: bool = False, title: str = None, text: str = None, activity_image: str = None,
                 activity_title: str = None, activity_subtitle: str = None, activity_text: str = None,
                 hero_image: HeroImage = None, facts: List[Fact] = None, actions: list = None):
        self._data = {}
        if start_group:
            self.set_start_group(start_group)
        if title is not None:
            self.set_title(title)
        if text is not None:
            self.set_text(text)
        if activity_image is not None:
            self.set_activity_image(activity_image)
        if activity_title is not None:
            self.set_activity_title(activity_title)
        if activity_subtitle is not None:
            self.set_activity_subtitle(activity_subtitle)
        if activity_text is not None:
            self.set_activity_text(activity_text)
        if hero_image is not None:
            self.set_hero_image(hero_image)
        if facts:
            self.add_facts(facts)
        if actions:
            self.add_potential_actions(actions)
        super().__init__()

    def set_start_group(self, start_group=True):
        self._data["startGroup"] = start_group

    def set_title(self, title: str):
        self._data["title"] = title

    def set_text(self, text: str):
        self._data["text"] = text

    def set_activity(self, image: str = None, title: str = None, subtitle: str = None, text: str = None):
        if image is not None:
            self.set_activity_image(image)
        if title is not None:
            self.set_activity_title(title)
        if subtitle is not None:
            self.set_activity_subtitle(subtitle)
        if text is not None:
            self.set_activity_text(text)

    def set_activity_image(self, image: str):
        self._data["activityImage"] = image

    def set_activity_title(self, title: str):
        self._data["activityTitle"] = title

    def set_activity_subtitle(self, subtitle: str):
        self._data["activitySubtitle"] = subtitle

    def set_activity_text(self, text: str):
        self._data["activityText"] = text

    def set_hero_image(self, hero_image: HeroImage):
        self._data["heroImage"] = hero_image.as_data()

    def add_facts(self, facts: List[Fact]):
        self._data.setdefault("facts", [])
        self._data["facts"].extend(self._get_items_list(facts))

    def add_fact(self, fact: Fact):
        self._data.setdefault("facts", [])
        self._data["facts"].append(fact.as_data())

    def add_potential_actions(self, actions: list):
        self._data.setdefault("potentialAction", [])
        self._data["potentialAction"].extend(self._get_items_list(actions))

    def add_potential_action(self, potential_action):
        self._data.setdefault("potentialAction", [])
        self._data["potentialAction"].append(potential_action.as_data())
